*  Move Block .02
*  By Adam Trionfo, based on a program by Eric Beckett.
*  July 20, 2010
*  
*  This is a re-written version of the Move Square program by Eric 
*  Beckett from the Aug, Sept, and Nov, 1984 issues of the APF
*  Imagination  Machine Newletter, "I-M 1 in a Million."  That
*  version required BASIC.  This version runs as a cartridge.
*  
*  This program is written to be run on the circa 1978 APF
*  MP-1000 computer game console which uses the
*  Motorola 6800 8-Bit CPU. 
*      
*  Program Description
*  -------------------  
*
*  The APF Menu gives one choice to the user: 1) RUN PROGRAM
*  
*  This program places a white block in the middle of the screen.
*  The user can control it using the Right Joystick.  The four
*  cardinal directions are supported-- diagnals are NOT supported.
*  
*  Version
*  -------
*  .02 - Added Light-Blue "Walls" to all four sides of the screen
*      - Added Boundary Checking (White Block can't move past "walls")
*
*  .01 - First Release  

*  Equates (Static)
    
MIDSCR    EQU  $02EF        ; Location of Middle of Screen
TOPLEFT   EQU  $0200        ; Top-Left Corner of Screen
TOPRIGHT  EQU  $021F        ; Top-Right Corner of Screen
LOWRLEFT  EQU  $03E0        ; Lower-Left Corner of Screen
NORTH     EQU  $4E          ; ASCII Capital N
SOUTH     EQU  $53          ; ASCII Capital S
EAST      EQU  $45          ; ASCII Capital E
WEST      EQU  $57          ; ASCII Capital W
BACKGRND  EQU  $80          ; ASCII of Background Color
WHITEBLK  EQU  $CF          ; ASCII of White Block
WALL      EQU  $DF          ; ASCII of Light-Blue Block ("Wall")

*  Equates (RAM Locations - Variables)

JOYDATA   EQU  $01F2        ; Joystick Data
NEWLOC    EQU  $0180        ; New Location of White Block (Two Bytes)
CURLOC    EQU  $0182        ; Cur Location of White Block (Two Bytes)
OLDLOC    EQU  $0184        ; Old Location of White Block (Two Bytes)
ENDCOLUM  EQU  $0186        ; End of Column (Two Bytes)
  
*  Equates (On-Board Routines)

FILLSCRN  EQU  $4296        ; Fill Screen Routine
READJOY2  EQU  $41D9        ; Read Right Joystick
          
          ORG   $8000       ; Start of APF Cartridge Area
          
*  The BIOS uses the first five bytes of the cartridge  

          FCB   $BB         ; Tell BIOS a cart is present
          FDB   MENUSTR     ; Points to Menu string on cartridge 
          FCB   $31         ; LSB 4 bits is # of Menu Choices
*                             Menu has one choice          
          FCB   $00         ; Checksum-type byte.  Must be $00 else
*                             BIOS restarts right after Init

*  After a menu choice is made from the APF Menu via the
*  BIOS, then control passes to the program here ($8005).  
 
          JSR   FILLSCRN    ; Fill Screen Background with Black      

**********************          
*                    *
*  Main Program Loop *
*                    *
**********************

          JSR   INITPROG    ; Initialize Program
MOVBLOCK  JSR   READSTIK    ; Read Joystick and Move Block    
          JMP   MOVBLOCK    ; Continue Running Program Forever 

*****************          
*               *
*  Subroutines  *
*               *
*****************

*  Initialize Program
*  ------------------
*  1) Fill top and bottom rows with Light-Blue (for "Wall")
*  2) Fill Left and Right side with Light-Blue (for "Wall")
*  3) Move the White Block to the middle of the screen.

INITPROG
*  Fill Top Row
          LDAA  #WALL       ; Set-Up Color for Walls
          LDX   #TOPLEFT    ; Prepare for Top-Line Fill Row
          JSR   FILLROW     ; Fill the Top Row with Light-Blue
*  Fill Bottom Row
          LDX   #LOWRLEFT   ; Prepare for Top-Line Fill Row
          JSR   FILLROW     ; Fill the Top Row with Light-Blue
*  Fill Left-Side Column
          LDX   #$03E0      ; Bottom of Column Address
          STX   ENDCOLUM    ; Store at End of Column
          LDX   #TOPLEFT    ; Top of Column Address
          JSR   FILLCOLM    ; Fill Vertical Column
*  Fill Right-Side Column
          LDX   #$03FF      ; Bottom of Column Address
          STX   ENDCOLUM    ; Store at End of Column
          LDX   #TOPRIGHT   ; Top of Column Address
          JSR   FILLCOLM    ; Fill Vertical Column
* Put White Block in Middle of Screen
          LDX   #MIDSCR     ; Middle of Screen
          STX   CURLOC      ; Current location of the White Block
          STX   OLDLOC      ; First Time through Old and Current are same                 
          LDAA  #WHITEBLK   ; Place ASCII White Block into Accum A 
          STAA  $00,X       ; Store White Block on Screen
          RTS

*  Has the Joystick been moved?
*  ----------------------------

READSTIK  JSR   READJOY2    ; Right Joystick Subroutine 
          BCS   JOY2USED    ; Branch if Joystick Used
          RTS               ; No Movement- Return and keep checking

*  Check which direction the joystick has been moved
*  -------------------------------------------------

* Check North
JOY2USED  LDAA  JOYDATA     ; Put Joystick Direction Data into Accum A 

          CMPA  #NORTH      ; Did Joystick Move North?
          BNE   CHKSOUTH    ; If not, then Check South Movement
          JMP   MOVNORTH    ; Direction was North, Branch

* Check South 
CHKSOUTH  CMPA  #SOUTH      ; Did Joystick Move South? 
          BNE   CHKEAST     ; If not, then Check East Movement
          JMP   MOVSOUTH    ; Direction was South, Branch

* Check East 
CHKEAST   CMPA  #EAST       ; Did Joystick Move East?
          BNE   CHKWEST     ; If not, then Check West Movement
          JMP   MOVEAST     ; Direction was East, Branch

* Check West
CHKWEST   CMPA  #WEST       ; Did Joystick Move West?
          BNE   NODIR       ; If not, then No Direction was Picked
          JMP   MOVWEST     ; Direction was West, Branch

NODIR     RTS               ; No Movement- Return and keep checking 

* Update White Block's Movement on the Screen
* -------------------------------------------

* White Block's East Movement
MOVEAST   LDX   CURLOC      ; Load Current Location of White Block 
          STX   OLDLOC      ; Location of White Block BEFORE Update
          INX               ; Move Block Location East
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's West Movement 
MOVWEST   LDX   CURLOC      ; Load Current Location of White Block
          STX   OLDLOC      ; Location of White Block BEFORE Update
          DEX               ; Move Block Location West
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's North Movement
MOVNORTH  LDX   CURLOC      ; Load Current Location of White Block
          STX   OLDLOC      ; Location of White Block BEFORE Update
          LDAA  #32D        ; Initialize Loop for 32 Times
BLOCKUP   DEX               ; Countdown Loop for "Up" Movement 
          DECA              ; Decrement "Up" Movement Loop 
          BNE   BLOCKUP     ; Has screen RAM moved "Up"
          JMP   DRAWBLOK    ; Jump to Draw White Block Subroutine

* White Block's South Movement
MOVSOUTH  LDX   CURLOC      ; Load Current Location of White Block
          STX   OLDLOC      ; Location of White Block BEFORE Update
          LDAA  #32D        ; Initialize Loop for 32 Times 
BLOKDOWN  INX               ; Countdown Loop for "Down" Movement 
          DECA              ; Decrement "Down" Movement Loop
          BNE   BLOKDOWN    ; Has screen RAM moved "Down"

*  Draw White Block - Draw Block at New Locaction.
*  ----------------
*  1) Check boundaries.  Has a wall been hit?
*  2) Draw White Block if no wall was hit

DRAWBLOK          
*  Boundary Check 
          STX   NEWLOC      ; Update New Location of White Block
          LDAA  $00,X       ; Load What is in Updated Location
          CMPA  #WALL       ; Is the Updated Location a Wall?
          BEQ   DELYLOOP    ; If Yes, then Don't Draw Block   
*  Draw Block if not a wall         
          LDX   OLDLOC      ; Not a Wall, so Write Background
          STAA  $00,X       ; Store Background Color onto screen
          LDX   NEWLOC      ; Get Current Location of White Block
          LDAA  #WHITEBLK   ; Place ASCII White Block into Accum A
          STAA  $00,X       ; Draw White Block to Screen
          STX   CURLOC      ; New Location is now Current Location

DELYLOOP  LDAB  #50D         ; Number of Outer Loops 
OUTLOOP   LDAA  #255D        ; Number of Inner Loops
INLOOP    DECA               ; Decrement Inner Loop
          BNE   INLOOP       ; All Inner Loops Complete?   
          DECB               ; Decrement Outer Loop
          BNE   OUTLOOP      ; All Outer Loops Complete?
          
          RTS                ; Check for More Joystick Movement
          
*  Fill a complete Horizontal Line with a character
*  Accum A = Character to fill row with
*        X = Where to Start Placement of character 

FILLROW   LDAB  #32D         ; Set-up Loop for one full line
FILLR     STAA  $00,X        ; Place a character on screen
          INX                ; Move Screen Location to Right
          DECB               ; Has the loops run 32 times?
          BNE   FILLR        ; Continue Filling row if not Zero
          RTS

*  Fill Column
*  -----------
*  
*  Fill a complete Vertical Column with a character
*  Accum A = Character to fill row with
*        X = Where to Start Placement of character

FILLCOLM         
          STAA  $00,X       ; Store Character onto screen
COLUMND1  LDAB  #32D        ; Initialize Loop for 32 Times
COLUMND2  INX               ; Countdown Loop for "Down" Movement 
          DECB              ; Decrement "Down" Movement Loop
          BNE   COLUMND2    ; Has Column Moved Down?
          STAA  $00,X       ; Store Character onto screen
          CPX   ENDCOLUM
          BNE   COLUMND1
          STAA  $00,X       ; Store Character onto screen
          RTS                    
          
 
*  Cartridge Menu String
*  ---------------------
*  
*  The data required by the BIOS for the main menu is here.

MENUSTR   FCB   $E9         ; 9 spaces before Program Name string
          FCC   "MOVE BLOCK  .02"  ; Program Name String
          FCB   $E8         ; 8 spaces after text (EOL)
          FCB   $E1         ; 1 spaces (Before Author)
          FCC   "BY ADAM TRIONFO, JULY 20, 2010"  ; Author
          FCB   $E3         ; 3 spaces (After Author)
          FCB   $E3         ; 3 spaces (Before Author)          
          FCC    "BASED ON ERIC BECKETT'S"
          FCB   $E7         ; 7 spaces (After Author)
          FCC    "BASIC/ML EXAMPLE FROM 1984"
          FCB   $FE         ; Another 30 spaces
          FCB   $E5         ; Another 5 spaces
          FCC   "1. RUN PROGRAM                  "
          FCB   $FF         ; Control Byte - End of String

CARTEND   FILL  $FF,$9000-CARTEND ; Pad with $FF for a 4K Cart

* End of Program
